"""
Module to import the project_GEMS.py 'configuration' file and perform
sanity checks plus a quick check on mesh generation.

Also callable as a stand-alone program, mainly to view the results
of the mesh generation.
"""

import os

from os.path import join, exists
from anuga.utilities.polygon import read_polygon, number_mesh_triangles

import project_GEMS


#-------------------------------------------------------------------------------
# Sanity checks - check that required files/directories, etc, exist.
#-------------------------------------------------------------------------------

# flag - we check many things and then don't proceed if anything wrong
sanity_error = False               # checked at bottom of this file

# Test that environment variables are defined.
if os.getenv(project_GEMS.ENV_INUNDATIONHOME) is None:
	print "Environment variable '%s' is not set" % project_GEMS.ENV_INUNDATIONHOME
	sanity_error = True


#-------------------------------------------------------------------------------
# Check filename strings that MUST be set
#-------------------------------------------------------------------------------

if not project_GEMS.gems_order_filename:
	print ("Sorry, ordering file parameter 'gems_order_filename' "
		   "isn't set")
	sanity_error = True

if not project_GEMS.landward_boundary_filename:
	print ("Sorry, landward bounding points parameter "
		   "'landward_boundary_filename' isn't set")
	sanity_error = True

#-------------------------------------------------------------------------------
# Directory Structure
#-------------------------------------------------------------------------------

# check folders generated from environment variables.
if not exists(project_GEMS.home):
	print "Sorry, data directory '%s' doesn't exist" % project_GEMS.home
	sanity_error = True
	
if not exists(project_GEMS.anuga_folder):
	print "Sorry, ANUGA directory '%s' doesn't exist" % project_GEMS.anuga_folder
	sanity_error = True

if not exists(project_GEMS.topographies_folder):
	print ("Sorry, topo directory '%s' doesn't exist"
		   % project_GEMS.topographies_folder)
	sanity_error = True

if not exists(project_GEMS.polygons_folder):
	print ("Sorry, polygon directory '%s' doesn't exist"
		   % project_GEMS.polygons_folder)
	sanity_error = True

if not exists(project_GEMS.boundaries_folder):
	print ("Sorry, boundaries directory '%s' doesn't exist"
		   % project_GEMS.boundaries_folder)
	sanity_error = True

if not exists(project_GEMS.output_folder):
	print "Sorry, outputs directory '%s' doesn't exist" % project_GEMS.output_folder
	sanity_error = True

if not exists(project_GEMS.gauges_folder):
	print "Sorry, gauges directory '%s' doesn't exist" % project_GEMS.gauges_folder
	sanity_error = True

if not exists(project_GEMS.event_folder):
	print ("Sorry, you must generate event %s."
		   % project_GEMS.event)
	sanity_error = True

#-------------------------------------------------------------------------------
# Determine type of run
#-------------------------------------------------------------------------------

if project_GEMS.setup == 'trial':
	print 'trial'
	project_GEMS.scale_factor = 100
	project_GEMS.time_thinning = 96
	project_GEMS.yieldstep = 240
elif project_GEMS.setup == 'basic': 
	print 'basic'
	project_GEMS.scale_factor = 4
	project_GEMS.time_thinning = 12
	project_GEMS.yieldstep = 120
elif project_GEMS.setup == 'final': 
	print 'final'
	project_GEMS.scale_factor = 1
	project_GEMS.time_thinning = 4
	project_GEMS.yieldstep = 60
elif project_GEMS.setup == 'storm_surge_final':
	print 'storm surge final'
	project_GEMS.scale_factor = 1
	project_GEMS.time_thinning = 4
	project_GEMS.yieldstep = 720	
else:
	print ("Sorry, you must set the 'setup' variable to one of:"
		   '   trial - coarsest mesh, fast\n'
		   '   basic - coarse mesh\n'
		   '   final - fine mesh, slowest\n'
		   '\n'
		   "'setup' was set to '%s'" % project_GEMS.setup)
	sanity_error = True

#-------------------------------------------------------------------------------
# Check for errors detected above.
#-------------------------------------------------------------------------------

if sanity_error:
	msg = 'You must fix the above errors before continuing.'
	raise Exception, msg
	
#-------------------------------------------------------------------------------
# Reading elevation clip polygon
#-------------------------------------------------------------------------------

project_GEMS.elevation_clip_box = read_polygon(join(project_GEMS.topographies_folder,
											 project_GEMS.elevation_clip_box_filename))

#-------------------------------------------------------------------------------
# Reading polygons and creating interior regions
#-------------------------------------------------------------------------------

# Create list of land polygons with initial conditions
project_GEMS.land_initial_conditions = []
for filename, MSL in project_GEMS.land_initial_conditions_filename:
	polygon = read_polygon(join(project_GEMS.polygons_folder, filename))
	project_GEMS.land_initial_conditions.append([polygon, MSL])

# Create list of interior polygons with scaling factor
project_GEMS.interior_regions = []
for filename, maxarea in project_GEMS.interior_regions_data:
	polygon = read_polygon(join(project_GEMS.polygons_folder, filename))
	project_GEMS.interior_regions.append([polygon,
									 maxarea*project_GEMS.scale_factor])

# Initial bounding polygon for data clipping 
project_GEMS.bounding_polygon = read_polygon(join(project_GEMS.polygons_folder,
											 project_GEMS.bounding_polygon_filename))
project_GEMS.bounding_maxarea = project_GEMS.bounding_polygon_maxarea*project_GEMS.scale_factor

# Estimate the number of triangles                     
trigs_min = number_mesh_triangles(project_GEMS.interior_regions,
								  project_GEMS.bounding_polygon,
								  project_GEMS.bounding_maxarea)

print 'min estimated number of triangles', trigs_min
